local _, ns = ...
local WK = ns.WhoKnows
WK.ItemStats = {}
WK.Itemcache = {}
-------------------------------------------------------
local print = WK.print
local debugPrint = WK.debugPrint
local dbg =  WK.debugPrintUI
-------------------------------------------------------
WK.Itemcache.ItemIds = {}
WK.Itemcache.ItemFrames = {}
WK.Itemcache.interval = 3
WK.Itemcache.timerON = false

--[[
GetItemInfoInstant
]]--

function WK.Itemcache:Add(itemId, interval)
	interval = interval or WK.Itemcache.interval
	if not self.ItemIds[itemId] then
		self.ItemIds[itemId] = true
		debugPrint("Adding " .. itemId .. " to WhoKnows.Itemcache...")
	end
	if not WK.Itemcache.timerON then
		debugPrint("making timer...")
		WK.Itemcache.timerON = true
		C_Timer.After(interval, WK.Itemcache.cTimer)
	end
end

function WK.Itemcache:Count()
	local c = 0
	for _ in pairs(self.ItemIds) do
		c = c + 1
	end
	return c
end

function WK.Itemcache.cTimer()
	local ic = WK.Itemcache
	if ic:Count() == 0 then return end
	debugPrint("looking up items in Itemcache")
	for k, v in pairs(ic.ItemIds) do
		ic.ItemIds[k] = nil -- remove the itemId from our table. If it still isn't ready then it will be re-added after the time interval
	--	WK.debugPrint("in cache table: "..WK.items[k].itemName.." with isComplete = "..tostring(WK.items[k].isIncomplete))
		WK.items[k] = nil --clears whatever we have in WhoKnows.items for that item
		if type(v) ~= "boolean" then
			WK.ItemGlow:OverlaySetup(v, k) -- forces a new lookup - if good then the overlay will update if not then it will end up back here.
		end
		ic.timerON = false
	end
end

WK.items = setmetatable({}, {
	__index = function(t, itemId)
		local item = WK.ItemStats:GetItemInfo(itemId)
		if not item then return end
		rawset(t, itemId, item)
		table.sort(t)
		return item
	end,
})

function WK.ItemStats:GetItemInfo(item)
	local itemId
	if type(item) == "number" then itemId = item
	elseif type(item) == "string" and WK.IsItemLink(item) then
		itemId = WK.GetItemIdFromHyperlink(item)
	end
	assert(itemId, "Error: WhoKnows.ItemStats:GetItemInfo did not receive a valid Item ID or Item link\n(itemID = " .. (tostring(itemId) or "nil") .. ")")
	--if not itemId then return end
	local I = {}
	I.itemName, I.itemLink, _, _, _, I.itemClass, I.profession = GetItemInfo(itemId)
	if (I.itemClass ~= "Recipe" and I.itemClass ~= "Miscellaneous") then print("BOPBOP") return end

	I.itemName = I.itemName:gsub("^.*:%s", "") --clean up itemName  - if colon present, remove it and everything preceding it
	I.itemName = I.itemName:gsub("Transmute", "Transmute:") -- spells have a colon after 'Transmute', but not in recipe name, so add it.

	I.itemName = I.itemName:lower()
	I.itemId = itemId
	I.isNotLearnableYet = nil
	I.isBOP = nil
	I.isBOPspell = nil --TODO: the spell itself is BOP
	I.isBOPItem = nil -- TODO: the item the spell makes is BOP
	I.isKnownBySelf = nil
	I.isBlueprint = nil
	I.reqToonLvl = nil
	I.reqSkillLvl = nil
	I.reqEngiSpec = nil
	I.needsPrereq = nil
	I.reqPreviousRank = nil
	I.enchantCatagory = nil
	I.isIncomplete = false
	I.spellTaughtId = 0

	local ttLines = WK.ToolTips:EnumerateTooltipLines(I.itemLink) --TODO: need to pass?
	WK.DebugTable.ttinfox = ttLines
	
	if not ttLines then
		debugPrint("Item " .. itemId .. ", " .. I.itemName .. " not ready - skipped")
		I.isIncomplete = true
		WK.Itemcache:Add(itemId)
		return
	end
	for i = 1, #ttLines do
		if ttLines[i]:match("Retrieving item information") then
			debugPrint(itemId .. " - Still retreiving")
			I.isIncomplete = true
			WK.Itemcache:Add(itemId)
		end
		I.isBOP = I.isBOP or (i <= 4 and (ttLines[i]:match("Binds when picked up") or ttLines[i]:match("Soulbound")))
		I.isBOPItem = I.isBOPItem or (i >= 5 and (ttLines[i]:match("Binds when picked up") or ttLines[i]:match("Binds when equipped")))
		I.isKnownBySelf = I.isKnownBySelf or ttLines[i]:match("^Already known")
		I.isBlueprint = I.isBlueprint or ttLines[i]:match("Garrison Blueprint")
		I.reqToonLvl = I.reqToonLvl or ttLines[i]:match('^Requires%sLevel%s(.*)')
		I.reqSkillLvl = I.reqSkillLvl or ttLines[i]:match("Requires%s" .. I.profession .. "%s[(](%d*)[)]")
		I.reqEngiSpec = I.reqEngiSpec or ttLines[i]:match("Requires%s(%w*)%sEngineer")
		I.needsPrereq = I.needsPrereq or (ttLines[i]:match("Must have learned") or ttLines[i]:match("^Requires%sWay%sof"))
		I.reqPreviousRank = I.reqPreviousRank or ttLines[i]:match("Requires Previous Rank")
		local reqClass = reqClass or ttLines[i]:match('^Classes:%s(.*)')
		if reqClass and not I.reqClass then
			reqClass = reqClass:gsub(",%s", ",")
			I.reqClass = { strsplit(",", reqClass) }
		end
		--track the # of times we loop since if a spell is BOP it will be listed within the 1st three lines but if the item the spell
		-- 	produces is BOP it will be listed as such later in the tooltip
		i = i + 1
	end
	I.reqSkillLvl = tonumber(I.reqSkillLvl) or -1
	I.reqToonLvl = tonumber(I.reqToonLvl) or -1
	I.spellTaughtId = WK.recipeDb:GetRecipeInfo(I.itemId) or -1
--	if I.spellTaughtId == -1 then
--		WK.recipeDb:GetRecipeInfo(I.itemId)
--		debugPrint(I.itemName.." : spellTaughtId: "..(I.spellTaughtId or "nil").." ID: "..(I.itemId or "nil"))
--	end
	if I.isKnownBySelf then I.isKnownBySelf = true end
	return I
end
