local _, ns = ...
local WK = ns.WhoKnows
local debugPrint = WK.debugPrint
local dbg =  WK.debugPrintUI

--WK.DebugTable.spell = {}
WK.ToolTips = {}

-- Make our own hidden tooltip to parse the data from because the one shown will have extra garbage in the way
WK.tt = CreateFrame("GameTooltip", "itemTooltip", nil, "GameTooltipTemplate");
WK.tt:SetOwner(WorldFrame, "ANCHOR_NONE");
local LibExtraTip = LibStub("LibExtraTip-1")
-------------------------------------------------------

function WK.TooltipSetItem(self, link)
	if (WK.lineAdded == true) then return; end
	WK.lineAdded = true
	WK.DebugTable.ttinfo = LibExtraTip:GetTooltipAdditional(self)
	if not link or not WK.IsItemLink(link) then
		WK.debugPrint( "cant get item link from item under mouse")
		return
	end
	if not WK.IsLearnableItem(link) then return end
	local itemId = WK.GetItemIdFromHyperlink(link)
	local tsr = WK.items[itemId]
	if not tsr then
		debugPrint("tsr = WK.items[itemId] failed for item# " .. itemId)
	end
	local toonSearchResults = {}
	if tsr.isIncomplete then
		toonSearchResults["Warning"] = {}
		toonSearchResults["Warning"]["msg"] = "Status unknown - waiting on server"
	else
		toonSearchResults = WK.PlayerStats:GetToonLearnedStatus(WK.items[itemId])
	end

	if toonSearchResults
		and WK.items[itemId].spellTaughtId
		and WK.items[itemId].spellTaughtId > 0 then-- if spellTaughtId isnt a spellID then the skill is from an NPC not an item
			if not WK.items[itemId].spellTaughtId or WK.items[itemId].spellTaughtId == -1 then
				debugPrint("spellTaughtId nil or -1 for "..(WK.items[itemId].." : "..(WK.items[itemName])))
			end
			WK.ToolTips:EditTooltip(self, toonSearchResults, WK.items[itemId][isBOP], itemId)
			WK.lineAdded = true
	elseif WK.debug == true then
		WK.ToolTips:EditTooltip(self, nil, WK.items[itemId][isBOP], itemId)
	end
end

function WK.ToolTips:HookToolTips()
	LibExtraTip:AddCallback({type = "item", callback = WK.TooltipSetItem})
	for _, frame in pairs { GameTooltip, ItemRefTooltip, ShoppingTooltip1, ShoppingTooltip2 } do
		if not LibExtraTip:IsRegistered(frame) then
			LibExtraTip:RegisterTooltip(frame)
			frame:HookScript("OnTooltipCleared", function(tooltip)
				WK.lineAdded = false
			end)
		end
	end
end

function WK.ToolTips:UnHookToolTips()
	 LibExtraTip:RemoveCallback(WK.TooltipSetItem)
end

local function EnumerateTooltipLines_helper(...)
	local results = {}
	for i = 1, select("#", ...) do
		local region = select(i, ...)
		if region and region:GetObjectType() == "FontString" then
			local text = region:GetText()
			if text then tinsert(results, text) end
		end
	end
	return results;
end

function WK.ToolTips:EnumerateTooltipLines(link)
	WK.tt:ClearLines()
	WK.tt:SetHyperlink(link)
	local results = EnumerateTooltipLines_helper(WK.tt:GetRegions())
	return results
end

-- Make our own hidden tooltip to parse the data from because the one shown will have extra garbage in the way
function WK.ToolTips:PrivateTooltip(link)
	if not WK.tt then
		WK.tt = CreateFrame("GameTooltip", "itemTooltip", nil, "GameTooltipTemplate");
		WK.tt:SetOwner(WorldFrame, "ANCHOR_NONE");
	end
	WK.tt:ClearLines()
	WK.tt:SetHyperlink(link)
	return WK.tt
end

function WK.ToolTips:PrivateTooltipByItemId(itemId)
	if not WK.tt then
		WK.tt = CreateFrame("GameTooltip", "itemTooltip", nil, "GameTooltipTemplate");
		WK.tt:SetOwner(WorldFrame, "ANCHOR_NONE");
	end
	WK.tt:ClearLines()
	WK.tt:SetItemByID(itemId)
	return WK.tt
end

--adds our findings to the tooltip
function WK.ToolTips:EditTooltip(tooltip, toonSearchResults, isBOP, itemId)
	LibExtraTip:AddLine(tooltip, "------------------", 1, .5, 0, true)
	LibExtraTip:AddLine(tooltip, "WhoKnows:", 1, .5, 0, true)
	if toonSearchResults and WK:tlength(toonSearchResults) > 0 then
		for k, v in pairs(toonSearchResults) do
			LibExtraTip:AddLine(tooltip, k .. v.msg, 0.78, 0.6, 0.43, true)
		end
	elseif isBOP then
		LibExtraTip:AddLine(tooltip, "Bind on Pickup Recipe", 0.76, 0.12, 0.23, true)
	end
	LibExtraTip:AddLine(tooltip, "------------------", 1, .5, 0, true)
	LibExtraTip:AddLine(tooltip, "Item ID: " .. tostring(itemId), 1, 1, 1, true)
	if WK.debug == true then tooltip:SetFrameStack(true) end
	WK.lineAdded = true
end
